from os import link
from YouTer.winds.class_templates.program_class import Program
from YouTer.winds.download_windows.save_winds.other_save_w import OtherSaveLocationWindow
from YouTer.winds.download_windows.link_w import LinkWindow

class SaveLocationWindow(Program):
    def __init__(self, command):
        self.command = command
        self.menu_options = {
            1: "Save on Desktop (Default)",
            2: "Enter custom save location",
            3: "Return to previous window"
        }

    def init_window(self):
        DEFAULT_FILE_PATH = "~/Desktop/%(title)s.%(ext)s"
        while True:
            self.clear()

            # Print Save location header
            print("Save location")
            self.line_breaker()

            self.print_menu_options(self.menu_options)

            user_input = self.user_input()

            if user_input == "1":
                link_w = LinkWindow(DEFAULT_FILE_PATH, self.command)
                link_w.init_window()
                break
            
            elif user_input == "2":
                other_save_w = OtherSaveLocationWindow(self.command)
                custom_file_path = other_save_w.init_window()

                link_w = LinkWindow(custom_file_path, self.command)
                link_w.init_window()
                break

            elif user_input == "3":
                break



from YouTer.winds.class_templates.program_class import Program
from YouTer.winds.download_windows.link_w import LinkWindow

class OtherSaveLocationWindow(Program):
    def __init__(self, command):
        self.window_message = "Make sure to enter a valid location.\n"
        self.command = command

    def init_window(self):
        while True:
            self.clear()

            print("Other Save Location")
            self.line_breaker()
            print(self.window_message)

            user_input = input("Save location: ")

            if user_input != "":
                file_path = user_input + "/%(title)s.%(ext)s"

                return file_path

            else:
                print("\nThe save location cannot be a empty space!\n")
                print("Press any key to try again...")

                input('') # empty input to hold loop



from YouTer.winds.class_templates.program_class import Program
from .youtubedl_w import YoutubeDlWindow

class LinkWindow(Program):
    def __init__(self, file_path, command):
        self.file_path = file_path
        self.command = command

    def init_window(self):
        self.clear()
        print("Enter Link")
        self.line_breaker()

        user_input = input("Enter video link (no spaces): ")

        final_command = f"{self.command} -o '{self.file_path}' {user_input}"

        youtubedl_w = YoutubeDlWindow(final_command)
        youtubedl_w.init_window()
        return user_input


import os
from .class_templates.program_class import Program

class AboutWindow(Program):
    def __init__(self, program_name, author_name, version):
        self.about_file_path = os.path.join(os.getcwd(), 'YouTer', 'texts', 'about.txt')
        self.author_name = author_name
        super().__init__(program_name, version)

    # Checks to see if the program is being run as package or script
    # in order to properly locate the .txt file about.txt

    # def check_file_path(self):
    #     # check if run as a package
    #     if os.path.exists(os.path.join(os.getcwd(), 'texts', 'about.txt')):
    #         self.about_file_path = os.path.join(os.getcwd(), 'ytdl-tgui', 'texts', 'about.txt')

    #     # check if run as a script
    #     elif os.path.exists(os.path.join(os.getcwd(), 'texts', 'about.txt')):
    #         self.about_file_path = os.path.join(os.getcwd(), 'texts', 'about.txt')

    #     else:
    #         print('Error locating file')
    #         input('\nPress any key to restart the process...')

    def read_about_file(self):
        about_text = []
        with open(self.about_file_path, 'r') as f:
            for line in f:
                about_text.append(line)
            f.close() # Remember to always close the opened files...
        return about_text

    # Init window
    def init_window(self):
        # print('start')
        # self.check_file_path()
        # print('end')

        self.clear()

        # About section header
        print("About Program")
        self.line_breaker()
        print(f"{self.program_name} - V{self.version}\n")

        # Print contents of about.txt
        file_lines = self.read_about_file()
        for line in file_lines:
            print(line)
        
        print(f'\nMade by {self.author_name} (2022).\n')
        self.line_breaker()

        input('Press any key to return to main menu...')


from numpy import save
from YouTer.winds.class_templates.program_class import Program
from YouTer.winds.download_windows.save_winds.save_w import SaveLocationWindow
from YouTer.winds.download_windows.video_winds.specific_quality_w import SpecificQualityOptionWindow

class VideoDownloadWindow(Program):
    def __init__(self):
        self.window_options = {
            1: "Best posible quality (Default)",
            2: "Select a specific quality",
            3: "Return to main menu"
        }

    def init_window(self):
        BEST_QUALITY_COMMAND = "-f 'bestvideo[ext=mp4]+bestaudio[ext=m4a]/mp4'"

        # Init window loop
        while True:
            self.clear()
            print("Download Video")
            self.line_breaker()

            self.print_menu_options(self.window_options)

            user_input = self.user_input()

            if user_input == "1":
                save_w = SaveLocationWindow(BEST_QUALITY_COMMAND)
                save_w.init_window()
                break
            
            elif user_input == "2":
                s_quality_w = SpecificQualityOptionWindow()
                special_quality = s_quality_w.init_window()

                save_w = SaveLocationWindow(special_quality)
                save_w.init_window()
                break
            
            elif user_input == "3":
                break


    


from YouTer.winds.class_templates.program_class import Program

class OptionsWindow(Program):
    def __init__(self):
        self.options_menu = {
            1: "Clear console per action",
            2: "Return to main menu"
        }
    
    # Get all options from options_menu dict
    def print_options(self):
        menu = self.options_menu
        for key in menu:
            print(f"{key}: {menu.get(key)}")
        print('\n')

    def init_window(self):
        self.clear()

        print("Options (Not working at the moment)")
        self.line_breaker()
        self.print_options()
        self.user_input()


import os
from YouTer.winds.class_templates.program_class import Program

class YoutubeDlWindow(Program):
    def __init__(self, command):
        self.command = command

    def init_window(self):
        self.clear()

        print("Download with youtube-dl")
        self.line_breaker()

        final_command = f"youtube-dl {self.command}"

        os.system("youtube-dl --version")

        print('\n')
        input("Press any key to restart the program...")




import os
import sys

class Program:
    def __init__(self, program_name, version):
        self.program_name = program_name
        self.version = version

    def line_breaker(self):
        print("=====================================\n")

    # prints the program header
    def program_header(self):
        print(f"{self.program_name} - {self.version}")
        self.line_breaker()

    # ask in console for use input
    def user_input(self):
        user_input = input("Your choice: ")
        return user_input

    def print_menu_options(self, dictionary):
        for key in dictionary:
            print(f"{key}: {dictionary.get(key)}")
        print("\n")

    def clear(self):
        # Windows
        if os.name == "nt":
            _ = os.system('cls')
        
        # Unix like, linux & macOS
        else:
            _ = os.system('clear')
    
    def exit(self):
        self.clear()
        sys.exit()


from .class_templates.program_class import Program

class MainWindow(Program):
    def __init__(self, program_name, version):
        super().__init__(program_name, version)
        self.menu_options = {
            1: "Download Video",
            2: "Download Audio",
            3: "Custom youtube-dl Command",
            4: "About",
            5: "Options",
            6: "Exit"
        }

    def init_program(self):
        self.clear()
        self.program_header()
        
        self.print_menu_options(self.menu_options)
        user_input = self.user_input()

        return user_input


from .class_templates.program_class import Program
from .download_windows.youtubedl_w import YoutubeDlWindow

class CustomYoutubeDlCommandWindow(Program):
    def __init__(self):
        pass

    def init_window(self):
        while True:
            self.clear()
            print("Custom Youtube-dl Command")
            self.line_breaker()
            
            print("Note: Enter your custom command after 'youtube-dl'\n")
            user_input = input("Command: youtube-dl ")

            if user_input != "":
                youtubedl_w = YoutubeDlWindow(user_input)
                youtubedl_w.init_window()
                break
            else:
                print("\nThe command can't be an empty string o.O\n")
                input('Press any key to try again... ')





from YouTer.winds.class_templates.program_class import Program

class SpecificQualityOptionWindow(Program):
    def __init__(self):
        self.menu_message = "Please note that the final file may not end up in .mp4 format\n"
        self.menu_options = {
            1: "144p",
            2: "240p",
            3: "360p",
            4: "480p",
            5: "720p",
            6: "1080p",
            7: "1440p",
            8: "2160p"
        }

    def get_quality_value(self, input):
        quality = []
        menu = self.menu_options
        for key in menu:
            if input == str(key):
                # input(' ')
                quality.append(self.menu_options.get(key))

        # Check if list has any value, cause if it has, then input was found in self.menu_options values.
        if len(quality) != 0:
            output_command = f"-f 'bestvideo[height={quality[0]}] +bestaudio[ext=m4a]/best[ext=mp4]/best'"
            return output_command
        else:
            return "error"



    def init_window(self):
        while True:
            first_part_command = "" # Reset value
            self.clear()

            print("Select Specific Quality")
            self.line_breaker()

            print(self.menu_message)

            self.print_menu_options(self.menu_options)

            user_input = self.user_input()

            # Purge user_inputs that are not valid
            if user_input == "":
                pass
            else:
                # Find out if user input is found in self.menu_options
                first_part_command = self.get_quality_value(user_input)

                # In case user_input is not found
                if first_part_command == "error":
                    pass # Continue with loop to restart process

                # Value found, returning youtube-dl command with specific quality
                else:
                    specific_quality_command = f"{first_part_command} --merge-output-format mp4"
                    return specific_quality_command